/*
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */

package holoedit.functions;

import holoedit.data.HoloSDIFdata;
import java.util.Vector;
import holoedit.gui.GestionPistes;

/** This class provides some methods whose names are easily understandable,
 * and that can be used directly in the textArea of the script editor window.*/
public final class ScriptEditorFunctions {
	
	/** ScriptEditorFunctions is non-instantiable.*/
	private ScriptEditorFunctions() {}
	
	/**
	 * Returns the holoSDIFdata that corresponds to the given description
	 * sdifdataDesc.
	 * @param gp do no pay attention to this parameter, just write "gp" (without quotes!).
	 * @param sdifdataDesc the description of an holoSDIFdata located in the score.
	 * @return The holoSDIFdata that corresponds to the given description.
	 */
	public static HoloSDIFdata getSDIFdata(GestionPistes gp, String sdifdataDesc){
		String[] split = sdifdataDesc.split(" - Track:", 2);
		holoedit.data.HoloTrack ht = gp.getTrack(Integer.valueOf(split[1]));
		return ht.getSDIFinstance(split[0]).getData();
	}
	
	/** Returns the values corresponding to the given time 
	 * if the given time is equal to a key of the holoSDIFdata's map.<br>
	 * If the given time is not equal to a key, it returns a linear interpolation
	 * of the values corresponding to the two closest key.<br>
	 * 'null' is returned if the sdifTreeMap is empty or if the time
	 * we look for is out of the map keyset range.*/
	public static Vector<Vector<Double>> getAllDataAtTime(HoloSDIFdata sdifdata, double time) {
		return sdifdata.getDataAtTime(time);
	}
	/** Returns the number of values contained by the sdifdata at the given time
	 * */
	public static int howManyDataAtTime(HoloSDIFdata sdifdata, double time) {
		return sdifdata.getDataAtTime(time).size();
	}
	
	/**
	 * Returns an array of double values corresponding to the frame times
	 * of the given sdifdata, and listed in natural order.
	 * @return An array containing all the times of the holoSDIFdata, listed in natural order.
	 */
	public static Object[] getTimes(HoloSDIFdata sdifdata) {
		return sdifdata.sdifTreeMap.keySet().toArray();
	}
	
	/**
	 * Returns the number of time samples that are contained
	 * by the given HoloSDIFdata.
	 * @return The number of time samples contained by the given holoSDIFdata.
	 */
	public static double getTimesCount(HoloSDIFdata sdifdata){
		return sdifdata.sdifTreeMap.size();
	}
	
	/**
	 * Returns the smallest value obtained from the sdifdata
	 * for the field corresponding to the given fieldIndex.
	 */
	public static double minFieldValue(HoloSDIFdata sdifdata, int fieldIndex) {
		return sdifdata.getHoloDataStat().minVector.get(fieldIndex);
	}
	
	/**
	 * Returns the largest value obtained from the sdifdata's field
	 * that corresponds to the given fieldIndex. 
	 */
	public static double maxFieldValue(HoloSDIFdata sdifdata, int fieldIndex) {
		return sdifdata.getHoloDataStat().maxVector.get(fieldIndex);
	}
	
	/**
	 * Returns the mean value obtained from the sdifdata's field
	 * that corresponds to the given fieldIndex.
	 */
	public static double meanFieldValue(HoloSDIFdata sdifdata, int fieldIndex) {
		return sdifdata.getHoloDataStat().meanVector.get(fieldIndex);
	}
	
	/**
	 * Returns the range (i.e. maxValue-minValue) of the sdifdata's field
	 * that corresponds to the given fieldIndex.
	 * @return A double value giving the range of the specified field of the specified holoSDIFdata.
	 */
	public static double rangeFieldValue(HoloSDIFdata sdifdata, int fieldIndex) {
		return sdifdata.getHoloDataStat().rangeVector.get(fieldIndex);
	}
	
	/**
	 * Returns the date at which the score selection begins.<br>
	 * Usage in a script :<br>
	 * int dateBegin = getBeginDate(gp); <br>
	 * @param gp do no pay attention to this parameter, just write "gp" (without quotes!).
	 * @return The date at which the score selection begins. 
	 */
	public static int getBeginDate(GestionPistes gp){		
		return gp.mainRef.counterPanel.getDate(1);
	}
	
	/**
	 * Returns the date at which the score selection is ending.<br>
	 * Usage in a script :<br>
	 * int endDate = getEndDate(gp);<br>
	 * @param gp do no pay attention to this parameter, just write "gp" (without quotes!).
	 * @return The date at which the score selection is ending. 
	 */
	public static int getEndDate(GestionPistes gp){		
		return gp.mainRef.counterPanel.getDate(2);
	}
	
	/**
	 * Returns the length of the score selection.<br>
	 * Usage in a script :<br>
	 * int dur = getDuration(gp);<br>
	 * @param gp do no pay attention to this parameter, just write "gp" (without quotes!).
	 * @return The length of the score selection. 
	 */
	public static int getDuration(GestionPistes gp){
		return gp.mainRef.counterPanel.getDate(2) - gp.mainRef.counterPanel.getDate(1);
	}
}
