/*
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */

package holoedit.functions;

import holoedit.data.HoloSDIFdata;
import java.util.Vector;
import holoedit.gui.GestionPistes;

/** This class provide methods whose names are easily understandable,
 * and that can be used directly in the textArea of the script editor window.*/
public final class ScriptEditorFunctions {
	
	/** ScriptEditorFunctions is non-instantiable.*/
	private ScriptEditorFunctions() {}
	
	/** Returns the values corresponding to the given time 
	 * if the given time is equal to a key of the holoSDIFdata's sdiftreeMap.\n
	 * If the given time is not equal to a key, it returns a linear interpolation
	 * of the values corresponding to the two closest key.\n
	 * 'null' is returned if the sdifTreeMap is empty or if the time
	 * we look for is out of the treeMap keyset range.*/
	public static Vector<Vector<Double>> getAllDataAtTime(HoloSDIFdata sdifdata, double time) {
		return sdifdata.getDataAtTime(time);
	}
	
	/**
	 * Returns an array of double values corresponding to the frame times
	 * of the sdifdata, and listed in natural order.
	 * @param sdifdata
	 * @return an array containing all the times of the holoSDIFdata, ranged in natural order.
	 */
	public static Object[] getAllTime(HoloSDIFdata sdifdata) {
		return sdifdata.sdifTreeMap.keySet().toArray();
	}
	
	/**
	 * Returns the smallest value obtained from the sdifdata
	 * for the field corresponding to the given fieldIndex. * 
	 * @param sdifdata
	 * @param fieldIndex
	 * @return the smallest value obtained from the given sdifdata for the specified field.
	 */
	public static double minFieldValue(HoloSDIFdata sdifdata, int fieldIndex) {
		return sdifdata.getHoloDataStat().minVector.get(fieldIndex);
	}
	
	/**
	 * Returns the largest value obtained from the sdifdata
	 * for the field corresponding to the given fieldIndex. * 
	 * @param sdifdata
	 * @param fieldIndex
	 * @return the largest value obtained from the given sdifdata for the specified field.
	 */
	public static double maxFieldValue(HoloSDIFdata sdifdata, int fieldIndex) {
		return sdifdata.getHoloDataStat().maxVector.get(fieldIndex);
	}
	
	/**
	 * Returns the mean value obtained from the sdifdata's field
	 * that corresponds to the given fieldIndex. * 
	 * @param sdifdata
	 * @param fieldIndex
	 * @return the mean value obtained from the given sdifdata for the specified field.
	 */
	public static double meanFieldValue(HoloSDIFdata sdifdata, int fieldIndex) {
		return sdifdata.getHoloDataStat().meanVector.get(fieldIndex);
	}
	
	/**
	 * Returns the range (i.e. maxValue-minValue) of the sdifdata's field
	 * that corresponds to the given fieldIndex. * 
	 * @param sdifdata
	 * @param fieldIndex
	 * @return a double value giving the range of the specified field of the specified holoSDIFdata.
	 */
	public static double rangeFieldValue(HoloSDIFdata sdifdata, int fieldIndex) {
		return sdifdata.getHoloDataStat().rangeVector.get(fieldIndex);
	}
	
	/**
	 * Returns the number of time samples that are contained
	 * by the given HoloSDIFdata.
	 * @param sdifdata
	 * @return the number of time samples contained by the given holoSDIFdata.
	 */
	public static double getTimesCount(HoloSDIFdata sdifdata){
		return sdifdata.sdifTreeMap.size();
	}
	
	/**
	 * Returns a double value with a positive sign, greater than or equal to 0.0 and less than 1.0.
	 * Returned values are chosen pseudorandomly with (approximately) uniform distribution from that range.
	 * @return a pseudorandom double greater than or equal to 0.0 and less than 1.0.
	 */
	public static double random(){
		return Math.random();
	}
	
	/**
	 * Returns the holoSDIFdata that corresponds to the given description
	 * sdifdataDesc.
	 * @param gp
	 * @param sdifdataDesc the description of an holoSDIFdata located in the score.
	 * @return the holoSDIFdata that corresponds to the given description.
	 */
	public static HoloSDIFdata getSDIFdata(GestionPistes gp, String sdifdataDesc){
		String[] split = sdifdataDesc.split(" - Track:", 2);
		holoedit.data.HoloTrack ht = gp.getTrack(Integer.valueOf(split[1]));
		return ht.getSDIFinstance(split[0]).getData();
	}
	
	/**
	 * Returns the date at which the score selection begins.
	 * Usage in scriptFile :
	 * int dateBegin = getBeginDate(gp);
	 * (do no pay attention to the 'gp' parameter).
	 * @param gp
	 * @return the date at which the score selection begins. 
	 */
	public static int getBeginDate(GestionPistes gp){		
		return gp.mainRef.counterPanel.getDate(1);
	}
	
	/**
	 * Returns the date at which the score selection is ending.
	 * Usage in scriptFile :
	 * int endDate = getEndDate(gp);
	 * (do no pay attention to the 'gp' parameter).
	 * @param gp
	 * @return the date at which the score selection is ending. 
	 */
	public static int getEndDate(GestionPistes gp){		
		return gp.mainRef.counterPanel.getDate(2);
	}
	
	/**
	 * Returns the length of the score selection.
	 * Usage in scriptFile :
	 * int dur = getDuration(gp);
	 * (do no pay attention to the 'gp' parameter).
	 * @param gp
	 * @return the length of the score selection. 
	 */
	public static int getDuration(GestionPistes gp){
		return gp.mainRef.counterPanel.getDate(2) - gp.mainRef.counterPanel.getDate(1);
	}
}
